@@ -2,18 +2,22 @@
 
 __author__ = """Dave St.Germain"""
 __email__ = 'dave@st.germa.in'
-__version__ = '0.2.0'
+__version__ = '0.6.0'
 
 import time
 from .relay import Manager, Relay
 
 
-async def get_anything(anything:str, relays=None, verbose=False, only_stored=True):
+async def get_anything(anything:str, relays=None, verbose=False, stream=False, origin='aionostr'):
     """
     Return anything from the nostr network
     anything: event id, nprofile, nevent, npub, nsec, or query
+
+    To stream events, set stream=True. This will return an asyncio.Queue to
+    retrieve events from
     """
     from .util import from_nip19
+
     query = None
     single_event = False
     if isinstance(anything, list):
@@ -24,11 +28,13 @@ async def get_anything(anything:str, relays=None, verbose=False, only_stored=Tru
     elif isinstance(anything, dict):
         query = anything
     elif anything.strip().startswith('{'):
+        from json import loads
         query = loads(anything)
-    elif anything.startswith(('nprofile', 'nevent', 'npub', 'nsec')):
+    elif anything.startswith(('nprofile', 'nevent', 'npub', 'nsec', 'nostr:')):
+        anything = anything.replace('nostr:', '', 1)
         obj = from_nip19(anything)
         if not isinstance(obj, tuple):
-            yield obj.hex()
+            return obj.hex()
         else:
             relays = obj[2] or relays
             if obj[0] == 'nprofile':
@@ -45,32 +51,48 @@ async def get_anything(anything:str, relays=None, verbose=False, only_stored=Tru
     if query:
         if not relays:
             raise NotImplementedError("No relays to use")
-        async with Manager(relays, verbose=verbose) as man:
-            async for event in man.get_events(query, single_event=single_event, only_stored=only_stored):
-                yield event
+
+        man = Manager(relays, verbose=verbose, origin=origin)
+        if not stream:
+            async with man:
+                return [event async for event in man.get_events(query, single_event=single_event, only_stored=True)]
+        else:
+            import asyncio
+            queue = asyncio.Queue()
+            async def _task():
+                async with man:
+                    async for event in man.get_events(query, single_event=single_event, only_stored=False):
+                        await queue.put(event)
+            asyncio.create_task(_task())
+            return queue
 
 
-async def add_event(relays, event:dict=None, private_key='', kind=1, pubkey='', content='', created_at=None, tags=None, verbose=False):
+async def add_event(relays, event:dict=None, private_key='', kind=1, pubkey='', content='', created_at=None, tags=None, direct_message='', verbose=False):
     """
     Add an event to the network, using the given relays
     event can be specified (as a dict)
     or will be created from the passed in parameters
     """
     if not event:
+        from nostr.key import PrivateKey
         from .event import Event
+        from .util import from_nip19
         created_at = created_at or int(time.time())
         tags = tags or []
-        from nostr.key import PrivateKey
         if not private_key:
             raise Exception("Missing private key")
 
         if private_key.startswith('nsec'):
-            from .util import from_nip19
             private_key = from_nip19(private_key).hex()
         prikey = PrivateKey(bytes.fromhex(private_key))
 
         if not pubkey:
             pubkey = prikey.public_key.hex()
+        if direct_message:
+            dm_pubkey = from_nip19(direct_message).hex() if direct_message.startswith('npub') else direct_message
+            tags.append(['p', dm_pubkey])
+            kind = 4
+            content = prikey.encrypt_message(content, dm_pubkey)
         event = Event(pubkey=pubkey, content=content, created_at=created_at, tags=tags, kind=kind)
         event.sign(prikey.hex())
         event_id = event.id
