@@ -29,6 +29,7 @@ def main(args=None):
 @main.command()
 @click.option('-r', 'relays', help='relay url', multiple=True, default=DEFAULT_RELAYS)
 @click.option('-s', '--stream', help='stream results', is_flag=True, default=False)
+@click.option('-v', '--verbose', help='verbose results', is_flag=True, default=False)
 @click.option('-q', '--query', help='query json')
 @click.option('--ids', help='ids')
 @click.option('--authors', help='authors')
@@ -39,7 +40,7 @@ def main(args=None):
 @click.option('--until', help='until')
 @click.option('--limit', help='limit')
 @async_cmd
-async def query(ids, authors, kinds, etags, ptags, since, until, limit, query, relays, stream):
+async def query(ids, authors, kinds, etags, ptags, since, until, limit, query, relays, stream, verbose):
     """
     Run a query once and print events
     """
@@ -69,11 +70,23 @@ async def query(ids, authors, kinds, etags, ptags, since, until, limit, query, r
     if not query:
         click.echo("some type of query is required")
         return -1
-    async for obj in get_anything(query, relays, only_stored=not stream):
-        click.echo(obj)
+    await _get(query, relays, verbose=verbose, stream=stream)
 
 
 
+async def _get(anything, relays, verbose=False, stream=False):
+    response = await get_anything(anything, relays, verbose=verbose, stream=stream)
+    if isinstance(response, str):
+        click.echo(response)
+    elif isinstance(response, asyncio.Queue):
+        while True:
+            event = await response.get()
+            click.echo(event)
+    else:
+        for event in response:
+            click.echo(event)
+
+
 @main.command()
 @click.argument("anything")
 @click.option('-r', 'relays', help='relay url', multiple=True, default=DEFAULT_RELAYS)
@@ -83,8 +96,7 @@ async def get(anything, relays, verbose, stream=False):
     """
     Get any nostr event
     """
-    async for obj in get_anything(anything, relays, verbose=verbose, only_stored=not stream):
-        click.echo(obj)
+    await _get(anything, relays, verbose=verbose, stream=stream)
 
 
 @main.command()
@@ -96,8 +108,9 @@ async def get(anything, relays, verbose, stream=False):
 @click.option('--pubkey', default='', help='public key')
 @click.option('--tags', default='[]', help='tags')
 @click.option('--private-key', default='', help='private key')
+@click.option('--dm', default='', help='pubkey to send dm')
 @async_cmd
-async def send(content, kind, created, tags, pubkey, relays, private_key, verbose):
+async def send(content, kind, created, tags, pubkey, relays, private_key, dm, verbose):
     """
     Send an event to the network
 
@@ -120,12 +133,41 @@ async def send(content, kind, created, tags, pubkey, relays, private_key, verbos
         kind=kind,
         content=content,
         tags=tags,
+        direct_message=dm,
         verbose=verbose,
     )
     click.echo(event_id)
     click.echo(to_nip19('nevent', event_id, relays))
 
 
+@main.command()
+@click.argument("anything")
+@click.option('-r', 'relays', help='relay url', multiple=True, default=DEFAULT_RELAYS)
+@click.option('-v', '--verbose', help='verbose results', is_flag=True, default=False)
+@click.option('-t', '--target', help='target relay', required=True)
+@async_cmd
+async def mirror(anything, relays, target, verbose):
+    """
+    Mirror a query from source relays to the target relay
+    """
+    if verbose:
+        click.echo(f'mirroring: {anything} from {relays} to {target}')
+    from . import Manager
+    async with Manager([target]) as man:
+        result_queue = await get_anything(anything, relays=relays, verbose=verbose, stream=True)
+        count = 0
+        while True:
+            event = await result_queue.get()
+            await man.add_event(event, check_response=True)
+            count += 1
+            if verbose:
+                click.echo(f'{event.id} from {event.pubkey}')
+            else:
+                if count % 100 == 0:
+                    click.echo(f'{count}...')
+        click.echo(f'{count} sent')
+
+
 @main.command()
 @click.argument("ntype")
 @click.argument("obj_id")
